--Include file for the use of complex numbers
--Written by Laurence Draper
--comments, questions? :- E-mail me vonfreudenhowser@rhodale.co.uk

------------------constants-------------------------------
include constant.e
------------------complex type----------------------------
global type complex(sequence num)
    return ( length(num) = 2 and atom (num[1]) and atom (num[2]))
end type

-------------------complex functions----------------------
global function conj(complex a) --returns the conjugate of a complex number a
    return {a[1],-a[2]}
end function

global function real(complex a) --returns the real part of a complex number
    return a[1]
end function

global function Imag(complex a) --returns the imaginary part of a complex number
    return a[2]
end function

global function i(complex a) --multiplies complex number a by i
    return {-a[2],a[1]}
end function

--------------------conversions---------------------------
global function complex_to_ModArg(complex a) --converts a complex number to its
					     --modulus and argument
        atom arg
        if a[1] = 0 then
	    return {a[2],pi/2*mod(a[2])}
        end if
        arg = arctan(a[2]/a[1])
        if arg >  pi then
	    arg = 2*pi-arg
        end if
        if arg < -pi then
	    arg = -2*pi-arg
        end if
        return {sqrt(a[1]*a[1]+a[2]*a[2]),arg}
end function

global function ModArg_to_complex(complex a) --converts a modulus and argument to a
				      --complex number
    return a[1]*{cos(a[2]),sin(a[2])}
end function


------------------complex operators-----------------------

global function complex_add(complex a, complex b) -- adds two complex numbers
    return {a[1]+b[1],a[2]+b[2]}
end function

global function complex_mult(complex a, complex b) --multiplies two complex numbers
    return {a[1]*b[1]-a[2]*b[2],a[1]*b[2]+a[2]*b[1]}
end function

global function complex_sub(complex a, complex b) --subtracts complex number b from a
    return {a[1]-b[1],a[2]-b[2]}
end function

global function complex_div(complex a,complex b) --divides complex number a by b
    return complex_mult(a,conj(b))/real(complex_mult(b,conj(b)))
end function

------------------complex logarithms----------------------

global function EI(complex a) --returns e to the power a, where a is complex
    return power(e,a[1])*{cos(a[2]),sin(a[2])}
end function

global function Log(complex a)
    a = complex_to_ModArg(a)
    return {log(a[1]),a[2]}
end function

global function LOG(complex a, complex base) --returns the complex log of a number
					     --to a complex base
    return complex_div(Log(a),Log(base))
end function

------------------complex Powers--------------------------
global function Sqrt(complex a)
    complex ret
    ret = {0,0}
    if a[2] = 0 then
        if a[1] >= 0 then
	    return {sqrt(a[1]),0}
        else
	    return {0, sqrt(-a[1])}
        end if
    end if
    a = complex_to_ModArg(a)

    return {power(a[1],1/2)*cos(a[2]/2),power(a[1],1/2)*sin(a[2]/2)}
end function

global function Power(complex a, complex b) --returns complex number a to the power
					    --of complex number b
    a = complex_to_ModArg(a)
    if b[2] = 0 then
        return power(a[1],b[1])*{cos(b[1]*a[2]),sin(b[1]*a[2])}
    end if
    return power(a[1],b[1])*E(-a[2]*b[2])*{cos(a[2]*b[1]+a[1]*ln(b[2])),
					   sin(a[2]*b[1]+a[1]*ln(b[2]))}
end function

------------------complex trig functions------------------
global function Cos(complex a)
    return complex_div(complex_add(EI(i(a)),EI(i(-a))),{2,0})
end function

global function Cosh(complex a)
    return complex_div(complex_add(EI(-a),EI(a)),{2,0})
end function

global function Sin(complex a)
    return complex_div(complex_sub(EI(i(a)),EI(i(-a))),{0,2})
end function

global function Sinh(complex a)
    return complex_div(complex_sub(EI(a),EI(-a)),{2,0})
end function

global function Tan(complex a)
    return complex_div(Sin(a),Cos(a))
end function

global function Tanh(complex a)
    return complex_div(Sinh(a),Cosh(a))
end function

global function Sec(complex a)
    return complex_div({1,0},Cos(a))
end function

global function Sech(complex a)
    return complex_div({1,0},Cosh(a))
end function

global function CoSec(complex a)
    return complex_div({1,0},Sin(a))
end function

global function CoSech(complex a)
    return complex_div({1,0},Sinh(a))
end function

global function Cot(complex a)
    return complex_div({1,0},Tan(a))
end function

global function Coth(complex a)
    return complex_div({1,0},Tanh(a))
end function

